/************************************************************************************
 * comRegDfns.h:  Defines bits in the registers of the RRIF dual port register block
 *                and the master-slave communication mailbox (in each slave IDRAM).
 *
 *  modifications:    
 *   - Removed the CommRegs structure from accessCommRegs.h, placed it here and
 *     deleted the file. Removed the comm. regs #defines form memoryPartitions.h
 *     and placed them here. This gives DAQ one file (this one) to include in their
 *     programs which, in conjunction with the ROD memory map structure (accessible
 *     via the MEMORY_MAP_REG), gives information on all the important areas of
 *     MDSP & SDSP memory.                                             26.02.04 dpsf
 ************************************************************************************/
#ifndef COM_REG_DFNS
#define COM_REG_DFNS

#include "txtBuffer.h"             /* pick up N_TXT_BUFFS */
#include "rodConfiguration.h"      /* pick up N_SDSP */

/* DSP Register Map Summary:
STATUS_REG_0     STATUS_REG_1     LIST_STATUS_REG  LIST_CNT_REG     
COMMAND_REG      TASK_STATE_REG   MEMORY_MAP_REG   TIME_REG
DIAGNOSTIC_REG   HCMD_REG         HSTATUS_REG_0    HSTATUS_REG_1
HSSTAT_REG_0     HSSTAT_REG_1     HSSTAT_REG_2     HSSTAT_REG_3      

TRAP_CMD_REG     TRAP_REQ_REG     TRAPSTAT_REG_0   TRAPSTAT_REG_1   
RESERVED_REG_0   RESERVED_REG_1   RESERVED_REG_2   RESERVED_REG_3  
RESERVED_REG_4   RESERVED_REG_5   RESERVED_REG_6   RESERVED_REG_7  
RESERVED_REG_8   LOOP_REG         INTR_DSP_HSHK    INTR_DSP_HSHK 
*/

typedef struct CommRegs {
	UINT32 status[2];
	UINT32 listStatus;
	UINT32 listCnt;
	
	UINT32 command;
	UINT32 taskState;
	UINT32 memoryMap;
	UINT32 time;
	
	UINT32 diagnostic;
	UINT32 hCmd;
	UINT32 hStat[2];
	
	UINT32 hSdspStat[4];
	
	UINT32 trapCmd;
	UINT32 trapReq;
	UINT32 trapStat[2];
	
	UINT32 reservedReg[9];
	UINT32 loop;

/* Inter-dsp communication registers: used for the handshake between master and
 * slave DSPs when one sends a list for the other to process.  These registers
 * reside in the IDRAM of each SDSP.  The addresses are known to both the master
 * and the slaves.  Note that the write register for the slave is the read register
 * for the master and vice versa. */
	#if (defined(I_AM_HOST))
		UINT32 privateDsp[2];
	#elif (defined(I_AM_MASTER_DSP))
		UINT32 interDspHshkWr, interDspHshkRd;
	#elif (defined(I_AM_SLAVE_DSP))
		UINT32 interDspHshkRd, interDspHshkWr;
	#endif

} CommRegs;

/* The host must know the starting address of the communication registers for
   the MDSP, so that it can find & use the memory map register: */
#if defined(I_AM_HOST)
	#define IDREGS_BASE  0x80000000
#endif

/* communication registers: used for the handshake for host to DSP primitive lists */
/* +0x00 */
#define STATUS_REG_0     (IDREGS_BASE)
#define STATUS_REG_2     ((STATUS_REG_0)    + 4)
#define LIST_STATUS_REG  ((STATUS_REG_2)    + 4)
#define LIST_CNT_REG     ((LIST_STATUS_REG) + 4)

/* +0x10 */
#define COMMAND_REG_0    ((LIST_CNT_REG)    + 4)
#define TASK_STATE_REG   ((COMMAND_REG_0)     + 4)
#define MEMORY_MAP_REG   ((TASK_STATE_REG)  + 4)
#define EXTRA_REG         ((MEMORY_MAP_REG)  + 4)

/* +0x20 */
#define DIAGNOSTIC_REG   ((EXTRA_REG)        + 4)
#define HCMD_REG         ((DIAGNOSTIC_REG)  + 4)
#define HSTATUS_REG_0    ((HCMD_REG)        + 4)
#define HSTATUS_REG_1    ((HSTATUS_REG_0)   + 4)

#define HCMD_STAT_REG_0   (HCMD_REG)
#define HSTAT_REG_0       (HSTATUS_REG_0)
#define HSTAT_REG_1       (HSTATUS_REG_1)
#define HCMD_STAT_REG_1   (EXTRA_REG)

/* +0x30 */
#define HSSTAT_REG_0     ((HSTATUS_REG_1)   + 4)
#define HSSTAT_REG_1     ((HSSTAT_REG_0)    + 4)
#define HSSTAT_REG_2     ((HSSTAT_REG_1)    + 4)
#define HSSTAT_REG_3     ((HSSTAT_REG_2)    + 4)

/* +0x40 */
#define TRAP_CMD_REG     ((HSSTAT_REG_3)    + 4)
#define TRAP_REQ_REG     ((TRAP_CMD_REG)    + 4)
#define TRAPSTAT_REG_0   ((TRAP_REQ_REG)    + 4)
#define TRAPSTAT_REG_1   ((TRAPSTAT_REG_0)  + 4)
/* dpsf temporary for backwards compatibility */
#define TRAP_CMD_STAT    (TRAP_CMD_REG)

/* +0x50 */
#define RESERVED_REG_0   ((TRAPSTAT_REG_1)  + 4)
#define RESERVED_REG_1   ((RESERVED_REG_0)  + 4)
#define RESERVED_REG_2   ((RESERVED_REG_1)  + 4)
#define RESERVED_REG_3   ((RESERVED_REG_2)  + 4)

/* +0x60 */
#define RESERVED_REG_4   ((RESERVED_REG_3)  + 4)
#define RESERVED_REG_5   ((RESERVED_REG_4)  + 4)
#define RESERVED_REG_6   ((RESERVED_REG_5)  + 4)
#define RESERVED_REG_7   ((RESERVED_REG_6)  + 4)

/* +0x70 */
#define RESERVED_REG_8   ((RESERVED_REG_7)  + 4)
#define LOOP_REG         ((RESERVED_REG_8)  + 4)

/* IDSP handshake registers: */
#if defined(I_AM_MASTER_DSP)
	#define INTR_DSP_HSHK_WR  ((LOOP_REG)         + 4)
	#define INTR_DSP_HSHK_RD  ((INTR_DSP_HSHK_WR) + 4)
#elif defined(I_AM_SLAVE_DSP)
	#define INTR_DSP_HSHK_RD  ((LOOP_REG)         + 4)
	#define INTR_DSP_HSHK_WR  ((INTR_DSP_HSHK_RD) + 4)
#endif


/* STATUS_REG_0 */
#define SR_RUNNING              0
#define SR_BUSY                 1
#define SR_EXECUTING            2
#define SR_PAUSED               3

#define SR_OUT_LIST_RDY         4
#define SR_DSP_ACK              5
#define SR_DMA_ACCESS_ACK       6
#define SR_DMA_ACCESS_ERR       7

#define SR_TXT_BUFF_NE(x)      (8  + (x))
#define SR_TXT_BUFF_PROC(x)    (12 + (x))

#define SR_NTASKS               16
#define SR_NTASKS_W             4

#define SR_IDLS_ACTIVE          20
#define SR_EVM                  21  /* SDSP only */
#define SR_IDLP_ACTIVE          21  /* MDSP only: for SDSP in CR */
#define SR_LISTERR              22
#define SR_LISTFATAL            23

#define SR_TXT_BUFF_OVF(x)      (24 +(x))

#define SR_DSP_WAITING          28
#define SR_INMEM_BUFF_ACK       29
#define SR_INIT_ERROR           30
#define SR_EMIF_ERROR           31

/* MDSP/SDSP Comm. Status Register (1): */
#define SR_SLVCOMM(x)             (x)
#define SR_HLTS(x)                (4 + (x))
#define SR_SLVATTEND               8

/* List Status Register: one INT32 field */
/* List Count Register: */
#define LCR_PRIM_INDEX      0
#define LCR_PRIM_INDEX_W    12
#define LCR_ERR_INDEX       12           
#define LCR_ERR_INDEX_W     12
#define LCR_LIST_INDEX      24
#define LCR_LIST_INDEX_W    8

/* Command Register: */
#define CR_IN_LIST_RDY            0
#define CR_PAUSE                  1
#define CR_RESUME                 2
#define CR_ABORT                  3
#define CR_TXT_BUFF_RR(x)        (4 + (x))
#define CR_RESET_INMEM_BUFF       8    /* MDSP: reset & clear the inmem buffer */
#define CR_DMA_ACCESS_REQ         10
#define CR_FLUSH_EVENTS           11   /* SDSPs */
#define CR_IDLP_ACTIVE            12   /* SDSPs */
#define CR_HISTO_OVERRIDE         13   /* Slave DSPs */
#define CR_ID                     28
#define CR_ID_W                   2   /* IDs for SDSPs, assigned by MDSP */

/* Task State Register */
#define TASK_STATE(x)            (4*x)
#define TASK_STATE_W              4

/* Memory Map Register: UINT32 * pointer */
/* Time & Loop Registers: one field */

/* Diagnostic Register: */
#define DR_DISP_ROD_REG_ID       0
#define DR_LOOPEVENT             0   //SDSP
#define DR_DISP_SERIAL_LOOPS     1
#define DR_USE_ROD_MASK_LUT      2
#define DR_HISTO_1               2   //SDSP
#define DR_HISTO_3               3   //SDSP
#define DR_DISP_MIRROR_CTIME     4

#define DR_STEP_CTRL             6
#define DR_STEP_TRIGGER          7
#define DR_SOFT_BC_RESET         9
#define DR_MAINLOOP_TOGGLE       12

#define DR_PULSE_PAUSE           14
//#define DR_TIMING_DISPLAY        15
#define DR_PULSEWAIT             15
#define DR_INFO_DISPLAY          16
#define DR_MODULE_CFG_LOOP       17
#define DR_SLAVE_WRITE_TEST      18
#define DR_CACHE_FLUSH           17 //SDSP
#define DR_CACHE_RANGE_FLUSH     18 //SDSP
#define DR_CACHE_TOGGLE          19 //SDSP

#define DR_ISR_DELAY             19
#define DR_DISP_LINK_DATA        20
#define DR_AUTO_STALL            21
#define DR_STAGE_PAUSE           22
#define DR_CAL_EVT_CAPTURE       23
#define DR_BIN_PAUSE             24
#define DR_NEW_FMTR              26
#define DR_DYN_DBG               27
#define DR_EFB_CNT_ENABLE        29
#define DR_SPEED_220             30
#define DR_RESET_FIFOS           31

/* Histogram Command & Status Registers */
#if (defined(I_AM_MASTER_DSP) || defined(I_AM_HOST))
	#define HCSR0_BIN          0
	#define HCSR0_BIN_W        8
	#define HCSR0_CHIP         8
	#define HCSR0_CHIP_W       8
	#define HCSR0_STAGE        16
	#define HCSR0_STAGE_W      16

	//#define HCSR0_CALLINE      8
	//#define HCSR0_CALLINE_W    5
	//#define HCSR0_CALLINE_EN   15
	//#define HCSR0_ERRORCNT     16
	//#define HCSR0_ERRORCNT_W   16

	#define HCSR1_AVGTRANS     0
	#define HCSR1_AVGTRANS_W   8
	#define HCSR1_AVGLEN       8
	#define HCSR1_AVGLEN_W     8
	#define HCSR1_AVGPROC      16
	#define HCSR1_AVGPROC_W    16

	#define HSR_SLV_W        16
	#define HSR0_SLV3        16
	#define HSR0_SLV2        0
	#define HSR1_SLV1        16
	#define HSR1_SLV0        0
#endif

/* Definitions of the SDSPs' command and status registers are different; 
   MDSP must know both. */
#define HCSR0_SLV_BIN          0
#define HCSR0_SLV_BIN_W        8
#define HCSR0_SLV_NEWBIN       8
#define HCSR0_SLV_CALLINE      9
#define HCSR0_SLV_CALLINE_W    5
#define HCSR0_SLV_CALLINE_EN   15
#define HCSR0_SLV_NEWCAL       17

#define HCSR1_SLV_BINEVT       0
#define HCSR1_SLV_BINEVT_W     32

#define HSR0_SLV_RDY          0
#define HSR0_SLV_EXP          1
#define HSR0_SLV_PROC         2
#define HSR0_SLV_DONE         3
#define HSR0_SLV_BINERR       8
#define HSR0_SLV_BINERR_W     8
#define HSR0_SLV_PROCTIME     16
#define HSR0_SLV_PROCTIME_W   16

#define HSR1_SLV_EVTS_REC     0
#define HSR1_SLV_EVTS_REC_W   32

/* Trap command-status register */
#define TCSR_TRAILER           0
#define TCSR_TRANSMIT          1
#define TCSR_HEADER            2
#define TCSR_ISR_ACTIVE        3

#define TCSR_DATA_ERROR        4
#define TCSR_HEADER_ERROR      5
#define TCSR_TRAILER_ERROR     6
#define TCSR_LINK_ERROR        7

#define TCSR_ERROR             8
#define TCSR_ISR_PENDING      11

#define TCSR_OVERFLOW_ERROR   14
#define TCSR_OVERFLOW         15

#define TCSR_ERR_COUNT        16
#define TCSR_ERR_COUNT_W       8

#define TCSR_EVT_COUNT        24
#define TCSR_EVT_COUNT_W       8

/* Trap status register 0 */
#define TSR0_EVT_WORD_CNT        0
#define TSR0_EVT_WORD_CNT_W     16

#define TSR0_IFRAME_TAIL        16
#define TSR0_IFRAME_TAIL_W      8
#define TSR0_XFRAME_TAIL        24
#define TSR0_XFRAME_TAIL_W      8

/* Trap status register 1 */
#define TSR1_IFRAME_HEAD        16
#define TSR1_IFRAME_HEAD_W      8
#define TSR1_XFRAME_HEAD        24
#define TSR1_XFRAME_HEAD_W      8

/* INTR_DSP_HSHK_WR and INTR_DSP_HSHK_RD */
/* The following bits are defined in each of these registers.  They are used for the
 * handshake between the master and a slave when either sends a list to the other.
 * The state machine is simpler than for host-originated lists, e.g. there is no
 * option to pause, it is processed all at once, and a simple status is given. */
#define INTR_DSP_IN_LIST_RDY     0
#define INTR_DSP_ABORT           1
#define INTR_DSP_ACK             2
#define INTR_DSP_OUT_LIST_RDY    3
#define INTR_DSP_LIST_ERR        4
#define INTR_DSP_LIST_FATAL      5

#endif

